///===========================================================================================================
//- Configuration
///===========================================================================================================
// Sound
const SOUND_HELD_SCRIPT = "Toucan.FlagMusic" // The SoundScript played after picking up the flag.
const SOUND_INTERRUPT_SCRIPT = "Game.KillStreak" // The SoundScript played when the player drops the flag or dies while carrying it.
const SOUND_CARRIED_DELAY = 5.0 // The delay it takes for the sound to start after picking up the flag.

// Self-destruction timer
const DESTRUCT_TIMER = 107.4 // The amount of time it takes the flag to count down to 0.
const DESTRUCT_RELAY = "flag_selfdestruct_relay" // The logic_relay to trigger once the timer counts down to 0.
const DESTRUCT_CASER = "time_ticker" // The logic_case entity to ping every decisecond. The timer's current time is provided in seconds with a single decimal point.

///===========================================================================================================
//- Variables
///===========================================================================================================
// Timer
DetTimer <- DESTRUCT_TIMER
DetTimerCountTime <- -1.0

// Carrier information
CurrentPlayerHeld <- null
LastPlayerHeld <- null
LastTeamHeld <- 0

// Sound
MusicIsPlaying <- false
MusicStartTime <- -1.0

// Performance binding
SetPropInt <- NetProps.SetPropInt.bindenv(NetProps)
SetPropBool <- NetProps.SetPropBool.bindenv(NetProps)

///===========================================================================================================
//- Functions
///===========================================================================================================
// Make sure certain assets are loaded
function Precache()
{
    self.PrecacheSoundScript(SOUND_HELD_SCRIPT)
    self.PrecacheSoundScript(SOUND_INTERRUPT_SCRIPT)
}

// Reset the flag when it spawns in
function OnPostSpawn()
{
    ResetStatistics()

    local gamerules = Entities.FindByClassname(null, "tf_gamerules")
    if ((gamerules) && (gamerules.IsValid()))
    {
        SetPropInt(gamerules, "m_nGameType", 1)
        SetPropBool(gamerules, "m_bPlayingHybrid_CTF_CP", true)
    }

    AddThinkToEnt(self, "Tick")
    self.ConnectOutput("OnPickup1", "PickedUp")
    self.ConnectOutput("OnDrop", "Dropped")
    self.ConnectOutput("OnCapture", "Captured")
    self.ConnectOutput("OnReturn", "ResetStatistics")
    self.ConnectOutput("OnReset", "Captured")
}

// Called when the flag is picked up, used to update carrier information
function PickedUp()
{
    if (activator == null || activator.IsValid() == false) { return }

    if ((activator.GetTeam() != LastTeamHeld) || (activator != LastPlayerHeld))
    {
        LastTeamHeld <- activator.GetTeam()
        LastPlayerHeld <- activator
        DetTimerUpdate()
    }

    CurrentPlayerHeld <- activator
    MusicStartTime <- Time() + SOUND_CARRIED_DELAY.tofloat()
    DetTimerCountTime <- Time() + 0.1
}

// Called when the flag is dropped
function Dropped()
{
    CurrentPlayerHeld <- null

    if (LastPlayerHeld && LastPlayerHeld.IsValid())
    {
        if (MusicIsPlaying == true)
        {
            MusicIsPlaying <- false
            LastPlayerHeld.StopSound(SOUND_HELD_SCRIPT)
            EmitSoundOnClient(SOUND_INTERRUPT_SCRIPT, LastPlayerHeld)
        }
    } else {
        LastPlayerHeld <- null
    }
}

// Called when the flag is captured
function Captured()
{
    if (LastPlayerHeld && LastPlayerHeld.IsValid())
    {
        if (MusicIsPlaying == true)
        {
            LastPlayerHeld.StopSound(SOUND_HELD_SCRIPT)
            EmitSoundOnClient(SOUND_INTERRUPT_SCRIPT, LastPlayerHeld)
        }
    }

    ResetStatistics()
}

function Tick()
{
    if (CurrentPlayerHeld && CurrentPlayerHeld.IsValid())
    {
        // Play the desired SoundScript after some time
        if ((MusicIsPlaying == false) && (Time() > MusicStartTime))
        {
            MusicIsPlaying <- true
            EmitSoundOnClient(SOUND_HELD_SCRIPT, CurrentPlayerHeld)
        }

        // Count down the self-destruction timer
        if (Time() > DetTimerCountTime)
        {
            DetTimerUpdate(DetTimer - 0.1)
            DetTimerCountTime <- Time() + 0.1
        }
    }

    return 0.0
}

// Updates the timer and the HUD or calls the timeout relay depending on the new time desired
function DetTimerUpdate(new_time = DESTRUCT_TIMER)
{
    // Round the number down to one decimal point
    new_time = floor((new_time + 0.01) * 10.0) / 10.0

    if (new_time > 0)
    {
        DetTimer <- new_time

        printl("Updated to: " + DetTimer.tostring())

        EntFire(DESTRUCT_CASER, "InValue", new_time.tostring(), 0.0, self)

        local display = ((DetTimer.tofloat() / DESTRUCT_TIMER.tofloat()) * 100).tointeger()
        SetPropInt(self, "m_nPointValue", display)
    } else {
        EntFire(DESTRUCT_RELAY, "Trigger", null, 0.0, self)
    }
}

// Clear variables
function ResetStatistics()
{
    // Reset sound variables
    MusicIsPlaying <- false

    // Reset carrier variables
    LastTeamHeld <- 0
    LastPlayerHeld <- null
    CurrentPlayerHeld <- null
}